//  
//  SoftBank X}[gtHT[rXJxTCg
//  t@XR[h12(Bluetooth)
//
//  Copyright(c) 2011 SOFTBANK MOBILE Corp. All rights reserved
//

package com.example.bluetoothsample;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.os.Handler;

public class BluetoothConnection {

	private static final String NAME = "BluetoothSample";
	private static final UUID MY_UUID = UUID
			.fromString("fa87c0d0-afac-11de-8a39-0800200c9a66");

	// ڑ󋵂萔
	public static final int NONE = 0;
	public static final int LISTEN = 1;
	public static final int CONNECTING = 2;
	public static final int CONNECTED = 3;

	private BluetoothAdapter mAdapter;
	private Handler mHandler;
	private int mState;

	private AcceptThread mAcceptThread;
	private ConnectThread mConnectThread;
	private ConnectedThread mConnectedThread;

	public BluetoothConnection(Context context, Handler handler) {
		mAdapter = BluetoothAdapter.getDefaultAdapter();
		mState = NONE;
		mHandler = handler;
	}

	// Ԃ̕ύXs܂
	private synchronized void setState(int state) {
		mState = state;
		mHandler.obtainMessage(BluetoothSample.MESSAGE_STATE, mState, -1)
				.sendToTarget();
	}

	// Ԃ擾܂
	public synchronized int getState() {
		return mState;
	}

	// T[o̐ڑ҂s܂
	public synchronized void start() {
		if (mConnectThread != null) {
			mConnectThread.cancel();
			mConnectThread = null;
		}
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}
		if (mAcceptThread == null) {
			mAcceptThread = new AcceptThread();
			mAcceptThread.start();
		}
		setState(LISTEN);
	}

	// NCAgƂĐڑv܂
	public synchronized void connect(BluetoothDevice device) {
		if (mState == CONNECTING) {
			if (mConnectThread != null) {
				mConnectThread.cancel();
				mConnectThread = null;
			}
		}
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}
		mConnectThread = new ConnectThread(device);
		mConnectThread.start();
		setState(CONNECTING);
	}

	// ڑ̏s܂
	public synchronized void connected(BluetoothSocket socket,
			BluetoothDevice device) {
		if (mConnectThread != null) {
			mConnectThread.cancel();
			mConnectThread = null;
		}
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}
		if (mAcceptThread != null) {
			mAcceptThread.cancel();
			mAcceptThread = null;
		}
		mConnectedThread = new ConnectedThread(socket);
		mConnectedThread.start();
		setState(CONNECTED);
	}

	// Bluetooth̐ؒfs܂
	public synchronized void stop() {
		if (mConnectThread != null) {
			mConnectThread.cancel();
			mConnectThread = null;
		}
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}
		if (mAcceptThread != null) {
			mAcceptThread.cancel();
			mAcceptThread = null;
		}
		setState(NONE);
	}

	// f[^̑Ms܂
	public void write(byte[] out) {
		ConnectedThread ct;
		synchronized (this) {
			if (mState != CONNECTED) {
				return;
			}
			ct = mConnectedThread;
		}
		ct.write(out);
	}

	// ڑ҂Xbh
	private class AcceptThread extends Thread {
		private BluetoothServerSocket serverSocket;

		public AcceptThread() {
			try {
				serverSocket = mAdapter.listenUsingRfcommWithServiceRecord(
						NAME, MY_UUID);
			} catch (IOException e) {
			}
		}

		public void run() {
			BluetoothSocket socket = null;
			while (mState != CONNECTED) {
				try {
					socket = serverSocket.accept();
				} catch (IOException e) {
					break;
				}
				if (socket != null) {
					synchronized (BluetoothConnection.this) {
						switch (mState) {
						case LISTEN:
						case CONNECTING:
							connected(socket, socket.getRemoteDevice());
							break;
						case NONE:
						case CONNECTED:
							try {
								socket.close();
							} catch (IOException e) {
							}
							break;
						}
					}
				}
			}
		}

		public void cancel() {
			try {
				serverSocket.close();
			} catch (IOException e) {
			}
		}
	}

	// ڑ̏sXbh
	private class ConnectedThread extends Thread {
		private BluetoothSocket socket;
		private InputStream input;
		private OutputStream output;

		public ConnectedThread(BluetoothSocket socket) {
			try {
				this.socket = socket;
				this.input = socket.getInputStream();
				this.output = socket.getOutputStream();
			} catch (IOException e) {
			}
		}

		public void run() {
			byte[] buf = new byte[1024];
			int bytes;
			while (true) {
				try {
					bytes = input.read(buf);
					mHandler.obtainMessage(BluetoothSample.MESSAGE_READ, bytes,
							-1, buf).sendToTarget();
				} catch (IOException e) {
					setState(LISTEN);
					break;
				}
			}
		}

		// f[^̑Ms܂
		public void write(byte[] buf) {
			try {
				output.write(buf);
			} catch (IOException e) {
			}
		}

		// LZ
		public void cancel() {
			try {
				socket.close();
			} catch (IOException e) {
			}
		}
	}

	// ڑvXbh
	private class ConnectThread extends Thread {
		private BluetoothDevice device;
		private BluetoothSocket socket;

		public ConnectThread(BluetoothDevice device) {
			try {
				this.device = device;
				this.socket = device.createRfcommSocketToServiceRecord(MY_UUID);
			} catch (IOException e) {
			}
		}

		// 
		public void run() {
			mAdapter.cancelDiscovery();
			try {
				socket.connect();
			} catch (IOException e) {
				setState(LISTEN);
				try {
					socket.close();
				} catch (IOException e2) {
				}
				BluetoothConnection.this.start();
				return;
			}
			synchronized (BluetoothConnection.this) {
				mConnectThread = null;
			}
			connected(socket, device);
		}

		// LZ
		public void cancel() {
			try {
				socket.close();
			} catch (IOException e) {
			}
		}
	}

}
